;;; alien.lisp --- CUDA Alien Functions

;; 

;;; Code:
(in-package :cuda)

(macrolet ((defcu (name &rest args)
             `(define-alien-routine ,name cu-result ,@args)))
  (defcu ("cuInit" cu-init) (flags unsigned-int))
  (defcu ("cuDeviceGet" cu-device-get) (device (* cu-device)) (ordinal int))
  (defcu ("cuDeviceGetCount" cu-device-get-count) (count (* int)))
  (defcu ("cuDeviceComputeCapability" cu-device-compute-capability) (major (* int)) (minor (* int)) (dev cu-device))
  (defcu ("cuDeviceGetName" cu-device-get-name) (name c-string) (len int) (dev cu-device))
  (defcu ("cuCtxCreate_v2" cu-ctx-create) (pctx (* cu-context)) (flags unsigned-int) (dev cu-device))
  (defcu ("cuCtxDestroy_v2" cu-ctx-destroy) (ctx cu-context))
  (defcu ("cuCtxSynchronize" cu-ctx-synchronize))
  (defcu ("cuDeviceTotalMem" cu-device-total-mem) (bytes (* size-t)) (dev int))
  (defcu ("cuMemAlloc_v2" cu-mem-alloc) (dptr (* cu-device-ptr)) (bytesize size-t))
  (defcu ("cuMemFree_v2" cu-mem-free) (dptr cu-device-ptr))
  (defcu ("cuMemHostRegister" cu-mem-host-register) (p (* t)) (byte-size size-t) (flags unsigned-int))
  (defcu ("cuMemHostUnregister" cu-mem-host-unregister) (p (* t)))
  (defcu ("cuMemcpyHtoD_v2" cu-memcpy-host-to-device) (dst cu-device-ptr) (src-host (* t)) (byte-count size-t))
  (defcu ("cuMemcpyHtoDAsync_v2" cu-memcpy-host-to-device-async) 
      (dst cu-device-ptr) (src-ptr (* t)) (byte-count size-t) (hstream cu-stream))
  (defcu ("cuMemcpyDtoH_v2" cu-memcpy-device-to-host) 
      (dst-host (* t)) (src-device cu-device-ptr) (byte-count size-t))
  (defcu ("cuMemcpyDtoHAsync_v2" cu-memcpy-device-to-host-async) 
      (dst-host (* t)) (src-device cu-device-ptr) (byte-count size-t) (hstream cu-stream))
  ;; requires fp traps disabled?
  (defcu ("cuModuleLoad" cu-module-load) (module (* cu-module)) (fname c-string))
  (defcu ("cuModuleUnload" cu-module-unload) (module cu-module))
  (defcu ("cuModuleGetFunction" cu-module-get-function) (hfunc (* cu-function)) (hmod cu-module) (name c-string))
  (defcu ("cuModuleGetGlobal_v2" cu-module-get-global)
      (dptr (* cu-device-ptr)) (bytes (* size-t)) (hmod cu-module) (name c-string))
  (defcu ("cuLaunchKernel" cu-launch-kernel) (f cu-function) 
    (grid-dim-x unsigned-int) (grid-dim-y unsigned-int) (grid-dim-z unsigned-int)
    (block-dim-x unsigned-int) (block-dim-y unsigned-int) (block-dim-z unsigned-int)
    (shared-mem-bytes unsigned-int) (hstream cu-stream) (kernel-params (* (* t))) (extra (* (* t))))
  (defcu ("cuEventCreate" cu-event-create) (phevent (* cu-event)) (flags unsigned-int))
  (defcu ("cuEventDestroy_v2" cu-event-destroy) (h-event cu-event))
  (defcu ("cuEventElapsedTime" cu-event-elapsed-time) (pmillis (* float)) (hstart cu-event) (hend cu-event))
  (defcu ("cuEventRecord" cu-event-record) (hevent cu-event) (hstream cu-stream))
  (defcu ("cuEventSynchronize" cu-event-synchronize) (hevent cu-event))
  (defcu ("cuStreamCreate" cu-stream-create) (phstream (* cu-stream)) (flags unsigned-int))
  (defcu ("cuStreamDestroy" cu-stream-destroy) (hstream cu-stream))
  (defcu ("cuStreamQuery" cu-stream-query) (hstream cu-stream))
  (defcu ("cuStreamSynchronize" cu-stream-synchronize) (hstream cu-stream))
  (defcu ("cuStreamWaitEvent" cu-stream-wait-event) (hstream cu-stream) (hevent cu-event) (flags unsigned-int)))
