;;; cli/clap/vars.lisp --- Clap Variables

;; 

;;; Code:
(in-package :cli/clap/vars)

(declaim (simple-string *cli-group-separator*))
(defparameter *cli-group-separator*
  "--"
  "A marker specifying the end of a unique group of CLI args.")

(defparameter *no-exit* nil
  "Indicate whether the WITH-CLI-HANDLERS form should exit on completion.")

(defvar *default-cli-def* 'defparameter)

(defvar *default-cli-class* 'cli
  "The name of the class of the top-level CLI object which will be
generated by the DEFINE-CLI macro.")

(declaim ((vector symbol) *cli-opt-kinds*))
(defvar *cli-opt-kinds* ;; make sure to keep this in sync with the list of parsers above
  (let ((kinds '(boolean string form list symbol keyword number file directory pathname)))
    (make-array (length kinds) :element-type 'symbol :initial-contents kinds)))

(defvar *cli* nil
  "Most recently used CLI object.
This symbol is bound in the body of the WITH-CLI macro.")

(defvar *args* nil
  "Current command arguments.
Bound for the lifetime of a DEFCMD function.")

(defvar *opts* nil
  "Current command options.
Bound for the lifetime of a DEFCMD function.")

(declaim (unsigned-byte *argc* *optc*))
(defvar *argc* 0
  "Current count of command arguments.
This value may be updated throughout the lifetime of a function defined with
DEFCMD.")

(defvar *optc* 0
  "Current count of command options.
This value may be updated throughout the lifetime of a function defined with
DEFCMD.")

(defvar *arg* nil
  "Current option argument.
Bound for the lifetime of a function defined with DEFOPT.")

(defvar *cli-package-table* (make-hash-table)
  "A hash table containing a mapping of package names to a list containing a
CLI object and two vectors: #(CLI CLI-CMDS CLI-OPTS).")
