;;; proto.lisp --- Skel Core Protocol

;; 

;;; Code:
(in-package :skel/core/proto)

;; ref: https://spdx.org/licenses/
(deftype license-designator () `(or null string pathname (member :mpl2 :wtfpl :lgpg :llgpl :gpl :mit :mit0)))

(deftype script-designator () '(member :bin :sh :bash :zsh :nu :lisp :python))

(deftype document-designator () '(member :org :txt :pdf :html :md))

(deftype contact-designator () '(or string (cons string string)))

(defgeneric sk-run (self)
  (:documentation "Run the object SELF."))
(defgeneric sk-new (self &rest args &key &allow-other-keys)
  (:documentation "Create a fresh instance of object SELF."))
(defgeneric sk-save (self)
  (:documentation "Save object SELF."))
(defgeneric sk-tangle (self &rest params &key &allow-other-keys)
  (:documentation "Tangle object self with params"))
(defgeneric sk-weave (self)
  (:documentation "Weave object self with params"))
(defgeneric sk-call (self arg)
  (:documentation "Call SELF with ARG."))
(defgeneric sk-call* (self &rest args)
  (:documentation "Call SELF with ARGS."))
(defgeneric sk-print (self &key &allow-other-keys)
  (:documentation "Print object SELF."))
(defgeneric sk-load (self &key &allow-other-keys)
  (:documentation "Load or reload object SELF."))
(defgeneric sk-load-component (kind form &optional path)
  (:documentation "Load a component of type KIND from provided FORM, producing an SK-COMPONENT
type. Usually calls SK-TRANSFORM or SK-CONVERT internally.

PATH is an optional directory pathname which will be merged with a filename
found in FORM. Defaults to *DEFAULT-PATHNAME-DEFAULTS*."))
(defgeneric sk-compile (self &key &allow-other-keys)
  (:documentation "Compile object SELF."))
(defgeneric sk-build (self &key &allow-other-keys)
  (:documentation "Build a skel-object."))
(defgeneric sk-install (self &key &allow-other-keys)
  (:documentation "Install a skel-object."))
(defgeneric sk-test (self &key &allow-other-keys)
  (:documentation "Test a skel-object."))
(defgeneric sk-deploy (self &key &allow-other-keys)
  (:documentation "Deploy a skel-object."))
(defgeneric sk-fetch (self &key &allow-other-keys)
  (:documentation "Fetch a skel-object."))
(defgeneric sk-pack (self &key &allow-other-keys)
  (:documentation "Pack a skel-object."))
(defgeneric sk-unpack (self &key &allow-other-keys)
  (:documentation "Unpack a skel-object."))
(defgeneric sk-bundle (self &key &allow-other-keys)
  (:documentation "Bundle a skel-object."))
(defgeneric sk-unbundle (self &key &allow-other-keys)
  (:documentation "Unbundle a skel-object."))
(defgeneric sk-transform (self other &key &allow-other-keys)
  (:documentation "Transform SELF to skel-object of type OTHER."))
(defgeneric sk-convert (self)
  (:documentation "Convert the object SELF."))
(defgeneric sk-read (self stream)
  (:documentation "Read and fill SELF with an object from STREAM."))
(defgeneric sk-read-file (self path)
  (:documentation "Read a PATH from the filesystem with SELF."))
(defgeneric sk-write (self stream)
  (:documentation "Write object SELF to STREAM."))
(defgeneric sk-writeln (self stream)
  (:documentation "Write a line from SELF to STREAM."))
(defgeneric sk-write-string (self)
  (:documentation "Return a string from SELF."))
(defgeneric sk-write-file (self &key path &allow-other-keys)
  (:documentation "Write from SELF to file PATH."))
(defgeneric sk-install-user-config (self config)
  (:documentation "Popu`late SELF with configuration object CONFIG."))
(defgeneric sk-find (name self &key &allow-other-keys)
  (:documentation "Find a rule with the given NAME in SELF."))
(defgeneric sk-register (self &key &allow-other-keys)
  (:documentation "Register sk-object SELF with the current Lisp system."))
